package com.perforce.workshop.tjuricek.p4oauth.controllers;

import com.perforce.workshop.tjuricek.p4oauth.config.UrlFactory;
import com.perforce.workshop.tjuricek.p4oauth.services.AccessTokenResult;
import com.perforce.workshop.tjuricek.p4oauth.services.CodeService;
import com.perforce.workshop.tjuricek.p4oauth.util.HttpServletRequestHelper;
import org.apache.commons.io.IOUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.function.Function;
import java.util.function.Supplier;

/**
 * Used to send the registered P4 token for the user back to the registered
 * redirect handler.
 */
public class FetchP4TokenGet implements ControllerMethod {

    private static final Logger logger = LogManager.getFormatterLogger(FetchP4TokenGet.class);

    public static final String FETCH_P4_TOKEN_PATH = "/p4token";

    private Function<HttpServletRequest, HttpServletRequestHelper> requestWrapper;
    private Supplier<CodeService> codeServiceSupplier;
    private Supplier<UrlFactory> urlFactory;

    public FetchP4TokenGet(Function<HttpServletRequest, HttpServletRequestHelper> requestWrapper,
                           Supplier<CodeService> codeServiceSupplier,
                           Supplier<UrlFactory> urlFactory) {
        this.requestWrapper = requestWrapper;
        this.codeServiceSupplier = codeServiceSupplier;
        this.urlFactory = urlFactory;
    }

    @Override
    public boolean matches(HttpServletRequest baseRequest) {
        HttpServletRequestHelper request = requestWrapper.apply(baseRequest);
        return request.pathStartsWith(FETCH_P4_TOKEN_PATH) &&
                request.hasMethod("GET") &&
                request.paramNotEmpty("client_id") &&
                request.paramNotEmpty("redirect_uri");
    }

    @Override
    public void handle(HttpServletRequest baseRequest, HttpServletResponse response) throws IOException, ServletException {
        HttpServletRequestHelper request = requestWrapper.apply(baseRequest);

        String login = getOrSendError(request, "client_id", response);
        if (login == null) return;

        String redirectUri = getOrSendError(request, "redirect_uri", response);
        if (redirectUri == null) return;

        String token = request.getBearerKey();

        String state = request.getParam("state").orElse(null);

        AccessTokenResult result = codeServiceSupplier.get().findAccessToken(login, redirectUri, token);

        if (result.hasError()) {
            logger.warn("unable to load the token, reason %s", result.getError());
            // I'm not sure how much information we should return in the response
            // here. Just going to return this general error and that's it
            response.sendError(HttpServletResponse.SC_BAD_REQUEST);
            return;
        }

        response.setContentType("text/plain");
        IOUtils.write(result.getPerforceToken(), response.getOutputStream());
    }
}
